/**
 * \file test.cpp
 *
 * Cipher nevű szorgalmi feladat tesztjei gtest_lite eszközeivel megvalósítva 
 * A szorgalmi feladat megoldásához ezt az állományt nem kell beadni (feltölteni).
 *
 * A ELKESZULT makró vezérli az egyes tesztesetek fordítását, ezzel lehetővé válik
 * hogy kisebb lépésekben teszteljünk.
 *
 */

#include <iostream>

#include "gtest_lite.h"
#include "memtrace.h"
#include "rstring.h"
#include "pool.h"

#define ELKESZULT 1

/* ELKESZULT makró:
    <= 3: RString tesztjei
    >= 10: Összes teszt
*/

int main() {
    GTINIT(std::cin);
    #if ELKESZULT > 0
        TEST(RString, konstruktorok) {
            RString a(100);
            EXPECT_STREQ("", a);
            EXPECT_EQ(100, a.capacity());
            EXPECT_EQ(0, a.size());
            
            RString b("alma");
            EXPECT_STREQ("alma", b);
            EXPECT_EQ(5, b.capacity());
            EXPECT_EQ(4, b.size());

            RString c("alma", 10);
            EXPECT_STREQ("alma", b);
            EXPECT_EQ(10, c.capacity());
            EXPECT_EQ(4, c.size());
        } END
    #endif

    #if ELKESZULT > 1
        TEST(RString, konstruktor kivetel) {
            RString a("alma", 5);
            EXPECT_STREQ("alma", a);
            EXPECT_EQ(5, a.capacity());
            EXPECT_EQ(4, a.size());

            try {
                EXPECT_THROW_THROW(RString b("alma", 4), const char* p);
            } catch (const char *p) {
            #ifdef CPORTA
                EXPECT_ENVCASEEQ("ORG_ID", p);
            #endif
            }
        } END
    #endif

    #if ELKESZULT > 2
        TEST(RString, ertekado_str) {
            RString a("alma", 20);
            a = "korte";
            EXPECT_STREQ("korte", a);
            EXPECT_EQ(20, a.capacity());
            EXPECT_EQ(5, a.size());
            a = "fa";
            EXPECT_STREQ("fa", a);
            EXPECT_EQ(20, a.capacity());
            EXPECT_EQ(2, a.size());
        } END
    #endif

    #if ELKESZULT > 3
        TEST(RString, cast) {
            RString a("alma", 20);
            char* str = a;
            strcpy(&str[4], "fa");
            EXPECT_STREQ("almafa", a);
            EXPECT_EQ(20, a.capacity());
            EXPECT_EQ(6, a.size());
        } END
    #endif


    #if ELKESZULT > 4
        TEST(StringPool, init) {
            StringPool pool(10, 20);
            EXPECT_EQ(10, pool.size());
            RString& ref = pool.acquire(15);
            EXPECT_EQ(20, ref.capacity());
            EXPECT_STREQ("", ref);
        } END
    #endif


    #if ELKESZULT > 5
        TEST(StringPool, free_size) {
            StringPool pool(2, 20);
            EXPECT_EQ(2, pool.free_size());
            RString& ref1 = pool.acquire(15);
            EXPECT_EQ(1, pool.free_size());
            RString& ref2 = pool.acquire(20);
            EXPECT_EQ(0, pool.free_size());
            EXPECT_EQ(20, ref1.capacity());
            EXPECT_EQ(20, ref2.capacity());
        } END
    #endif

    #if ELKESZULT > 6
        TEST(StringPool, create_new_obj) {
            StringPool pool(1, 20);
            RString& ref = pool.acquire(15);
            EXPECT_EQ(0, pool.free_size());
            RString& newobj = pool.acquire(10);
            EXPECT_EQ(0, pool.free_size());
            EXPECT_EQ(2, pool.size());
            EXPECT_EQ(10, newobj.capacity());
        } END
    #endif

    #if ELKESZULT > 7
        TEST(StringPool, release) {
            StringPool pool(1, 20);
            RString& ref = pool.acquire(15);
            EXPECT_EQ(1, pool.size());
            EXPECT_EQ(0, pool.free_size());
            ref = "alma";
            pool.release(ref);
            EXPECT_EQ(1, pool.free_size());
            EXPECT_EQ(1, pool.size());
        } END
    #endif



    if (ELKESZULT < 10)
      ADD_FAILURE() << "\nNem futott minden teszteset!" << std::endl;

    GTEND(std::cerr);
    return 0;
}
